@@ -118,6 +118,29 @@ void voip::send_prack_message(context_t ctx, tid_t tid, msg_t msg)
     eXosip_unlock(ctx);
 }
 
+bool voip::make_options_response(context_t ctx, tid_t tid, int status, msg_t *msg)
+{
+    if(!msg)
+        return false;
+
+    *msg = NULL;
+    eXosip_lock(ctx);
+    eXosip_options_build_answer(ctx, tid, status, msg);
+    if(!*msg) {
+        eXosip_unlock(ctx);
+        return false;
+    }
+    return true;
+}
+
+void voip::send_options_response(context_t ctx, tid_t tid, int status, msg_t msg)
+{
+    if(!msg)
+        eXosip_lock(ctx);
+    eXosip_options_send_answer(ctx, tid, status, msg);
+    eXosip_unlock(ctx);
+}
+
 bool voip::make_answer_response(context_t ctx, tid_t tid, int status, msg_t *msg)
 {
     if(!msg)
@@ -463,6 +486,28 @@ voip::call_t voip::send_invite_request(context_t ctx, msg_t msg)
     return rtn;
 }
 
+bool voip::make_options_response(context_t ctx, tid_t tid, int status, msg_t *msg)
+{
+    if(!msg)
+        return false;
+    *msg = NULL;
+    eXosip_lock();
+    eXosip_options_build_answer(tid, status, msg);
+    if(!*msg) {
+        eXosip_unlock();
+        return false;
+    }
+    return true;
+}
+
+void voip::send_options_response(context_t ctx, tid_t tid, int status, msg_t msg)
+{
+    if(!msg)
+        eXosip_lock();
+    eXosip_options_send_answer(tid, status, msg);
+    eXosip_unlock();
+}
+
 bool voip::make_answer_response(context_t ctx, tid_t tid, int status, msg_t *msg)
 {
     if(!msg)
@@ -485,7 +530,6 @@ void voip::send_answer_response(context_t ctx, tid_t tid, int status, msg_t msg)
     eXosip_unlock();
 }
 
-
 void voip::send_request_message(context_t ctx, msg_t msg)
 {
     if(!msg)
@@ -784,3 +828,24 @@ void voip::show(msg_t msg)
     }
 }
 
+void voip::server_accepts(voip::msg_t msg)
+{
+    osip_message_set_header(msg, ACCEPT, "application/sdp, text/plain");
+}
+
+void voip::server_allows(voip::msg_t msg)
+{
+    osip_message_set_header(msg, ALLOW, "INVITE,ACK,CANCEL,OPTIONS,INFO,REFER,MESSAGE,SUBSCRIBE,NOTIFY,REGISTER,PRACK");
+    osip_message_set_header(msg, ALLOW_EVENTS, "talk, hold, refer");
+}
+
+void voip::server_requires(voip::msg_t msg, const char *txt)
+{
+    osip_message_set_require(msg, txt);
+}
+
+void voip::server_supports(voip::msg_t msg, const char *txt)
+{
+    osip_message_set_supported(msg, txt);
+}
+
