@@ -96,9 +96,9 @@ public:
          * @param host of remote server.
          * @param port of remote server (0 = unspecified/srv).
          * @param addr buffer to use.
-         * @return true if resolved.
+         * @return schema assocated with uri, or NULL if not found.
          */
-        virtual bool resolve(const char *uri , struct sockaddr_storage *addr);
+        virtual const char *resolve(const char *uri , struct sockaddr_storage *addr);
 
         /**
          * Notify plugins a user registration has been expired or released.
