@@ -50,8 +50,10 @@ public:
     static bool resolve(const char *sipuri, char *buffer, size_t size);
     static bool rewrite(const char *sipuri, char *buffer, size_t size);
     static void serviceid(const char *sipuri, char *buffer, size_t size);
+    static bool server(struct sockaddr *address, char *buffer, size_t size);
     static bool userid(const char *sipuri, char *buffer, size_t size);
     static bool hostid(const char *sipuri, char *buffer, size_t size);
+    static unsigned short portid(const char *sipuri);
     static void identity(struct sockaddr *address, char *buffer, const char *user, size_t size);
     static void publish(const char *uri, char *buffer, const char *user, size_t size);
 };
