@@ -26,10 +26,10 @@
 #include <ctype.h>
 
 #ifdef  EXOSIP_API4
-#define EXOSIP_CONTEXT  stack::sip.context
-#define OPTION_CONTEXT  stack::sip.context,
-#define EXOSIP_LOCK     eXosip_lock(stack::sip.context);
-#define EXOSIP_UNLOCK   eXosip_unlock(stack::sip.context);
+#define EXOSIP_CONTEXT  stack::sip.out_context
+#define OPTION_CONTEXT  stack::sip.out_context,
+#define EXOSIP_LOCK     eXosip_lock(stack::sip.out_context);
+#define EXOSIP_UNLOCK   eXosip_unlock(stack::sip.out_context);
 #else
 #define EXOSIP_CONTEXT
 #define OPTION_CONTEXT
@@ -74,9 +74,9 @@ public:
         void update(void);
         bool expire(Socket::address& addr);
         bool refresh(Socket::address& addr, time_t expires, const char *target_contact);
-        unsigned setTargets(Socket::address& addr);
-        unsigned addTarget(Socket::address& via, time_t expires, const char *contact, const char *policy, struct sockaddr *peer);
-        unsigned setTarget(Socket::address& via, time_t expires, const char *contact, const char *policy, struct sockaddr *peer);
+        unsigned setTargets(Socket::address& addr, voip::context_t context);
+        unsigned addTarget(Socket::address& via, time_t expires, const char *contact, const char *policy, struct sockaddr *peer, voip::context_t context);
+        unsigned setTarget(Socket::address& via, time_t expires, const char *contact, const char *policy, struct sockaddr *peer, voip::context_t context);
         void addContact(const char *id);
         void addPublished(const char *id);
         void addRoute(const char *pat, unsigned pri, const char *prefix, const char *suffix);
@@ -394,7 +394,10 @@ private:
     unsigned invite_expires;
 
 public:
-    voip::context_t context;
+    voip::context_t out_context;
+    voip::context_t tcp_context;
+    voip::context_t udp_context;
+    voip::context_t tls_context;
 
     static stack sip;
 
@@ -527,7 +530,7 @@ public:
     static void logging(MappedRegistry *rr, const char *reason);
     static void registration(int id, modules::regmode_t mode);
     static bool authenticate(int id, const char *realm);
-    static struct sockaddr *resolve(const char *uri, struct sockaddr_storage *addr);
+    static const char *resolve(const char *uri, struct sockaddr_storage *addr);
     static const char *referLocal(MappedRegistry *rr, const char *target, char *buffer, size_t size);
     static const char *referRemote(MappedRegistry *rr, const char *target, char *buffer, size_t size);
     static bool checkId(const char *id);
@@ -587,7 +590,7 @@ private:
     service::keynode *routed;
     registry::mapped *reginfo;
     MappedRegistry *accepted;
-    eXosip_event_t *sevent;
+    voip::event_t sevent;
     bool activated;
     char binding[MAX_URI_SIZE];
     char buffer[MAX_URI_SIZE];
@@ -599,17 +602,17 @@ private:
     char requesting[MAX_URI_SIZE];
     Socket::address via_address, request_address, contact_address;
     stack::session *session;
-    osip_header_t *header;
+    voip::hdr_t header;
     long header_expires;
-    osip_via_t *via_header;
-    osip_from_t *from;
-    osip_to_t *to;
-    osip_uri_t *uri;
+    voip::via_t via_header;
+    voip::from_t from;
+    voip::to_t to;
+    voip::uri_t uri;
     const char *via_host, *contact_host;
     unsigned via_hops;
     unsigned via_port, from_port, contact_port;
     destination_t destination;
-    voip::context_t source_context, target_context;
+    voip::context_t context;
 
     char *sip_realm;
     osip_proxy_authenticate_t *proxy_auth;
@@ -617,7 +620,7 @@ private:
 
     enum {CALL, MESSAGE, REGISTRAR, NONE} authorizing;
 
-    thread();
+    thread(voip::context_t ctx);
 
     static void wait(unsigned count);
 
