@@ -78,6 +78,8 @@ cidr(acl, ifaddr(addr, id), id)
     active = true;
     String::set(netname, sizeof(netname), id);
 
+    shell::log(DEBUG3, "adding policy %s for %s", id, addr);
+
     if(eq(id, "world") || eq(id, "any") || eq(id, "nat")) {
         if(eq(id, "nat") || eq(id, "any"))
             String::set(netname, sizeof(netname), "*");
@@ -472,8 +474,7 @@ failed:
         EXOSIP_UNLOCK
         return;
     }
-    osip_message_set_header(msg, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-    osip_message_set_header(msg, ALLOW_EVENTS, "talk, hold, refer");
+    voip::server_allows(msg);
     osip_message_set_header(msg, "Referred-By", source->identity);
     eXosip_call_send_request(OPTION_CONTEXT did, msg);
     target->state = session::REFER;
@@ -519,8 +520,7 @@ void stack::infomsg(session *source, eXosip_event_t *sevent)
         snprintf(type, sizeof(type), "%s", ct->type);
     osip_message_set_content_type(msg, type);
     osip_message_set_body(msg, body->body, strlen(body->body));
-    osip_message_set_header(msg, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-    osip_message_set_header(msg, ALLOW_EVENTS, "talk, hold, refer");
+    voip::server_allows(msg);
     eXosip_call_send_request(OPTION_CONTEXT did, msg);
     EXOSIP_UNLOCK
 }
@@ -722,25 +722,18 @@ void stack::start(service *cfg)
         iface = sip_iface;
 
     thread *thr;
-    unsigned thidx = 0;
     shell::log(DEBUG1, "starting sip stack v%d; %d maps and %d threads at priority %d",
         ver, mapped_calls, threading, priority);
-#ifdef  EXOSIP_API4
-    sip.context = eXosip_malloc();
-#endif
-    eXosip_init(EXOSIP_CONTEXT);
 
+    tcp_context = udp_context = tls_context = out_context = NULL;
+
+#ifdef  EXOSIP_API4
+    sip.out_context = eXosip_malloc();
+#endif
     mapped_array<MappedCall>::create(control::env("callmap"), mapped_calls);
     if(!sip)
         shell::log(shell::FAIL, "calls could not be mapped");
     initialize();
-#ifdef  AF_INET6
-    if(sip_family == AF_INET6) {
-        eXosip_enable_ipv6(1);
-        if(!iface)
-            iface = "::0";
-    }
-#endif
 
 #ifdef  HAVE_TLS
     if(sip_tlsmode) {
@@ -771,34 +764,54 @@ void stack::start(service *cfg)
 #else
     Socket::family(sip_family);
 #endif
-
-    if(eXosip_listen_addr(OPTION_CONTEXT sip_protocol, iface, sip_port, sip_family, sip_tlsmode)) {
-#ifdef  AF_INET6
-        if(!iface && sip_family == AF_INET6)
-            iface = "::0";
-#endif
-        if(!iface)
-            iface = "*";
-        shell::log(shell::FAIL, "sip cannot bind interface %s, port %d", iface, sip_port);
-    }
-    #ifdef  AF_INET6
-    if(!iface && sip_family == AF_INET6)
-        iface = "::0";
-#endif
-    if(!iface)
-        iface = "*";
-
-    shell::log(shell::DEBUG1, "binding interface %s, port %d", iface, sip_port);
-
     osip_trace_initialize_syslog(TRACE_LEVEL0, (char *)"sipwitch");
-    eXosip_set_user_agent(OPTION_CONTEXT agent);
+
+    if(sip_protocol == IPPROTO_TCP) {
+        voip::create(&tcp_context, agent, sip_family);
+        voip::create(&udp_context, agent, sip_family);
+        out_context = tcp_context;
+    }
+    else {
+        voip::create(&udp_context, agent, sip_family);
+        voip::create(&tcp_context, agent, sip_family);
+        out_context = udp_context;
+    }
+
+#ifdef  HAVE_TLS
+    voip::create(&tls_context, agent, family);
+#endif
 
 #if defined(EXOSIP2_OPTION_SEND_101) && !defined(EXOSIP_API4)
     eXosip_set_option(EXOSIP_OPT_DONT_SEND_101, &send101);
 #endif
 
-    while(thidx++ < threading) {
-        thr = new thread();
+    threading = 0;
+    if(udp_context) {
+        ++threading;
+        if(!voip::listen(udp_context, IPPROTO_UDP, iface, sip_port))
+            shell::log(shell::FAIL, "cannot listen port %u for udp", sip_port);
+        else
+            shell::log(shell::NOTIFY, "listening port %u for udp", sip_port);
+        thr = new thread(udp_context);
+        thr->start(priority);
+    }
+
+    if(tcp_context) {
+        ++threading;
+        if(!voip::listen(tcp_context, IPPROTO_TCP, iface, sip_port))
+            shell::log(shell::FAIL, "cannot listen port %u for tcp", sip_port);
+        shell::log(shell::NOTIFY, "listening port %u for tcp", sip_port);
+        thr = new thread(tcp_context);
+        thr->start(priority);
+
+    }
+
+    if(tls_context) {
+        ++threading;
+        if(!voip::listen(tls_context, IPPROTO_TCP, iface, sip_port, true))
+            shell::log(shell::FAIL, "cannot listen port %u for tls", sip_port + 1);
+        shell::log(shell::NOTIFY, "listening port %u for tls", sip_port + 1);
+        thr = new thread(tls_context);
         thr->start(priority);
     }
 
@@ -1244,6 +1257,50 @@ int stack::inviteRemote(stack::session *s, const char *uri_target, const char *d
     unsigned icount = 0;
     time_t now;
     struct sockaddr_storage peering, abuf;
+    voip::context_t context = stack::sip.out_context;
+    const char *out_target = uri_target;
+    char rewrite[MAX_URI_SIZE];
+    const char *schema = server::resolve(out_target, &abuf);
+
+    if(!schema) {
+        if(eq(uri_target, "tcp:", 4))
+            schema = "tcp";
+        else if(eq(uri_target, "udp:", 4))
+            schema = "udp";
+        else if(eq(uri_target, "sips:", 5))
+            context = stack::sip.tls_context;
+    }
+
+    if(schema) {
+        while(*uri_target && *uri_target != ':' && *uri_target != '@')
+            ++uri_target;
+        if(*uri_target == ':')
+            ++uri_target;
+        else
+            uri_target = out_target;
+        target = (struct sockaddr *)&abuf;
+
+        if(eq(schema, "tcp")) {
+            context = stack::sip.tcp_context;
+            schema = "sip";
+        }
+        else if(eq(schema, "udp")) {
+            context = stack::sip.udp_context;
+            schema = "sip";
+        }
+        else if(eq(schema, "sips"))
+            context = stack::sip.tls_context;
+
+        // special schema rewrite for plugin based schema references    
+        snprintf(rewrite, sizeof(rewrite), "%s:%s", schema, uri_target);
+        uri_target = rewrite;
+    }
+
+    // if resolver target not supported, we fallback to internal defaults
+    if(!context) {
+        target = NULL;
+        context = stack::sip.out_context;
+    }
 
     time(&now);
 
@@ -1258,7 +1315,6 @@ int stack::inviteRemote(stack::session *s, const char *uri_target, const char *d
 
     // default if no target route lookup...
 
-    target = server::resolve(uri_target, &abuf);
     if(!target) {
         if(!port)
             port = 5060;
@@ -1303,9 +1359,8 @@ int stack::inviteRemote(stack::session *s, const char *uri_target, const char *d
 
     divert(call, invite);
 
-    osip_message_set_header(invite, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-    osip_message_set_header(invite, ALLOW_EVENTS, "talk, hold, refer");
-    osip_message_set_supported(invite, "100rel,replaces,timer");
+    voip::server_allows(invite);
+    voip::server_supports(invite, "100rel,replaces,timer");
 
     if(digest && s->reg) {
         char *authbuf = new char[1024];
@@ -1557,9 +1612,8 @@ int stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t de
 
         divert(call, invite);
 
-        osip_message_set_header(invite, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-        osip_message_set_header(invite, ALLOW_EVENTS, "talk, hold, refer");
-        osip_message_set_supported(invite, "100rel,replaces,timer");
+        voip::server_allows(invite);
+        voip::server_supports(invite, "100rel,replaces,timer");
 
         if(call->expires) {
             snprintf(expheader, sizeof(expheader), "%ld", (long)(call->expires - now));
