@@ -42,7 +42,7 @@ static char *remove_quotes(char *c)
     return o;
 }
 
-thread::thread() : DetachedThread(stack::sip.stacksize)
+thread::thread(voip::context_t ctx) : DetachedThread(stack::sip.stacksize)
 {
     to = NULL;
     from = NULL;
@@ -50,6 +50,7 @@ thread::thread() : DetachedThread(stack::sip.stacksize)
     routed = NULL;
     reginfo = NULL;
     session = NULL;
+    context = ctx;
 }
 
 void thread::publish(void)
@@ -1104,8 +1105,8 @@ void thread::send_reply(int error)
     case CALL:
         eXosip_call_build_answer(OPTION_CONTEXT sevent->tid, error, &reply);
         if(reply != NULL) {
-            if(stack::sip_protocol == IPPROTO_UDP)
-                osip_message_set_require(reply, "100rel");
+            if(context == stack::sip.udp_context)
+                voip::server_requires(reply, "100rel");
             stack::siplog(reply);
             eXosip_call_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
         }
@@ -1116,8 +1117,8 @@ void thread::send_reply(int error)
     case MESSAGE:
         eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, error, &reply);
         if(reply != NULL) {
-            if(stack::sip_protocol == IPPROTO_UDP)
-                osip_message_set_require(reply, "100rel");
+            if(context == stack::sip.udp_context)
+                voip::server_requires(reply, "100rel");
             stack::siplog(reply);
             eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
         }
@@ -1212,7 +1213,10 @@ bool thread::authenticate(void)
 
     if(stack::sip.restricted) {
         if(!getsource() || !access || !String::ifind(stack::sip.restricted, access->getName(), ",; \t\n")) {
-            if(via_host)
+            if(access && via_host && !String::ifind(stack::sip.restricted, access->getName(), ",; \t\n"))
+                shell::log(shell::NOTIFY, "restricted %s from <%s> for %s:%u",
+                    access->getName(), stack::sip.restricted, via_host, via_port);
+            else if(via_host)
                 shell::log(shell::NOTIFY, "rejecting restricted %s from %s:%u", auth->username, via_host, via_port);
             else
                 shell::log(shell::NOTIFY, "rejecting restricted %s", auth->username);
@@ -1303,42 +1307,27 @@ void thread::challenge(void)
         "Digest realm=\"%s\", nonce=\"%s\", algorithm=%s",
                 registry::getRealm(), nonce, registry::getDigest());
 
-    EXOSIP_LOCK
     switch(authorizing) {
     case REGISTRAR:
-        eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, &reply);
-        if(reply != NULL) {
-            osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
-            osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-            osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
-            stack::siplog(reply);
-            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, reply);
-        }
-        break;
     case MESSAGE:
-        eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, &reply);
-        if(reply != NULL) {
+        if(voip::make_response_message(context, sevent->tid, SIP_UNAUTHORIZED, &reply)) {
             osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
-            osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-            osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
+            voip::server_allows(reply);
             stack::siplog(reply);
-            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, reply);
+            voip::send_response_message(context, sevent->tid, SIP_UNAUTHORIZED, reply);
         }
         break;
     case CALL:
-        eXosip_call_build_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, &reply);
-        if(reply != NULL) {
-            osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-            osip_message_set_header(reply, ALLOW_EVENTS, "talk, hold, refer");
+        if(voip::make_answer_response(context, sevent->tid, SIP_UNAUTHORIZED, & reply)) {
             osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
+            voip::server_allows(reply);
             stack::siplog(reply);
-            eXosip_call_send_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, reply);
+            voip::send_answer_response(context, sevent->tid, SIP_UNAUTHORIZED, reply);
         }
         break;
     case NONE:
         break;
     }
-    EXOSIP_UNLOCK
 }
 
 bool thread::getsource(void)
@@ -1477,8 +1466,7 @@ reply:
             snprintf(temp, sizeof(temp), "%lu", (long)registry::getExpires());
             osip_message_set_expires(reply, temp);
         }
-        osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-        osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
+        voip::server_allows(reply);
         stack::siplog(reply);
         eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
     }
@@ -1581,22 +1569,18 @@ reply:
             shell::debug(3, "querying %s", reguri->username);
         else
             shell::debug(3, "query rejected for %s; error=%d", reguri->username, error);
-        EXOSIP_LOCK
-        eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, error, &reply);
-        if(reply != NULL) {
+        if(voip::make_response_message(context, sevent->tid, error, &reply)) {
             if(error == SIP_OK) {
                 snprintf(buftemp, sizeof(buftemp), "<%s:%s@%s>",
                     stack::getScheme(), reguri->username, binding);
                 osip_message_set_contact(reply, buftemp);
             }
-            osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-            osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
+            voip::server_allows(reply);
             stack::siplog(reply);
-            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
+            voip::send_response_message(context, sevent->tid, error, reply);
         }
         else
-            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
-        EXOSIP_UNLOCK
+            voip::send_response_message(context, sevent->tid, SIP_BAD_REQUEST, NULL);
         return;
     }
 
@@ -1654,9 +1638,9 @@ void thread::reregister(const char *contact, time_t interval)
     refresh = reginfo->refresh(contact_address, expire, contact);
     if(!refresh) {
         if(reginfo->type == MappedRegistry::USER)
-            count = reginfo->addTarget(contact_address, expire, contact, network, (struct sockaddr *)&peering);
+            count = reginfo->addTarget(contact_address, expire, contact, network, (struct sockaddr *)&peering, context);
         else
-            count = reginfo->setTarget(contact_address, expire, contact, network, (struct sockaddr *)&peering);
+            count = reginfo->setTarget(contact_address, expire, contact, network, (struct sockaddr *)&peering, context);
     }
     if(refresh)
         shell::debug(2, "refreshing %s for %ld seconds from %s:%u", getIdent(), (long)interval, contact_host, contact_port);
@@ -1683,10 +1667,7 @@ void thread::reregister(const char *contact, time_t interval)
         }
     }
 reply:
-    EXOSIP_LOCK
-    eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, answer, &reply);
-    if(reply != NULL) {
-
+    if(voip::make_response_message(context, sevent->tid, answer, &reply)) {
         if(answer == SIP_OK) {
             if(reginfo->ext) {
                 snprintf(buftemp, sizeof(buftemp), "<%s:%d@%s>;expires=%ld",
@@ -1698,14 +1679,12 @@ reply:
                 stack::getScheme(), reginfo->userid, binding, (long)interval);
             osip_message_set_contact(reply, buftemp);
         }
-        osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-        osip_message_set_header(reply, ALLOW_EVENTS, "talk, hold, refer");
+        voip::server_allows(reply);
         stack::siplog(reply);
-        eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, answer, reply);
+        voip::send_response_message(context, sevent->tid, answer, reply);
     }
     else
-        eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
-    EXOSIP_UNLOCK
+        voip::send_response_message(context, sevent->tid, SIP_BAD_REQUEST, NULL);
     if(reginfo && reginfo->is_user() && answer == SIP_OK)
         messages::update(identity);
 }
@@ -1744,19 +1723,16 @@ void thread::getDevice(registry::mapped *rr)
 
 void thread::options(void)
 {
-    osip_message_t *reply = NULL;
+    voip::msg_t reply = NULL;
 
-    EXOSIP_LOCK
-    if(eXosip_options_build_answer(OPTION_CONTEXT sevent->tid, SIP_OK, &reply) == 0) {
-        osip_message_set_header(reply, ACCEPT, "application/sdp, text/plain");
-        osip_message_set_header(reply, ALLOW, "INVITE,ACK,CANCEL,OPTIONS,INFO,REFER,MESSAGE,SUBSCRIBE,NOTIFY,REGISTER,PRACK");
-        osip_message_set_header(reply, ALLOW_EVENTS, "talk, hold, refer");
+    if(voip::make_options_response(context, sevent->tid, SIP_OK, &reply)) {
+        voip::server_accepts(reply);
+        voip::server_allows(reply);
         stack::siplog(reply);
-        eXosip_options_send_answer(OPTION_CONTEXT sevent->tid, SIP_OK, reply);
+        voip::send_options_response(context, sevent->tid, SIP_OK, reply);
     }
     else
-        eXosip_options_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
-    EXOSIP_UNLOCK
+        voip::send_options_response(context, sevent->tid, SIP_BAD_REQUEST, NULL);
 }
 
 void thread::shutdown(void)
@@ -1764,7 +1740,6 @@ void thread::shutdown(void)
     shutdown_flag = true;
     while(active_count)
         Thread::sleep(50);
-    eXosip_quit(EXOSIP_CONTEXT);
     while(shutdown_count < startup_count)
         Thread::sleep(50);
 }
@@ -1817,7 +1792,7 @@ void thread::run(void)
         identbuf[0] = 0;
 
         if(!shutdown_flag)
-            sevent = eXosip_event_wait(OPTION_CONTEXT 0, stack::sip.timing);
+            sevent = voip::get_event(context, stack::sip.timing);
 
         activated = false;
         accepted = NULL;
@@ -1828,12 +1803,15 @@ void thread::run(void)
 
         if(shutdown_flag) {
             shell::log(DEBUG1, "stopping event thread %d", instance);
+            voip::release(context);
             ++shutdown_count;
             return; // exits thread...
         }
 
-        if(!sevent)
+        if(!sevent) {
+            voip::automatic_action(context);
             continue;
+        }
 
         ++active_count;
         shell::debug(2, "sip: event %d; cid=%d, did=%d, instance=%d",
